@@ -374,14 +374,11 @@ eab_select_source (ESourceRegistry *registry,
 }
 
 gchar *
-eab_suggest_filename (const GSList *contact_list)
+eab_suggest_filename (EContact *contact)
 {
 	gchar *res = NULL;
 
-	g_return_val_if_fail (contact_list != NULL, NULL);
-
-	if (!contact_list->next) {
-		EContact *contact = E_CONTACT (contact_list->data);
+	if (contact) {
 		gchar *string;
 
 		string = e_contact_get (contact, E_CONTACT_FILE_AS);
@@ -597,8 +594,10 @@ eab_transfer_contacts (ESourceRegistry *registry,
 	destination = eab_select_source (
 		registry, source, desc, NULL, last_uid, window);
 
-	if (!destination)
+	if (!destination) {
+		g_slist_free_full (contacts, g_object_unref);
 		return;
+	}
 
 	if (strcmp (last_uid, e_source_get_uid (destination)) != 0) {
 		g_free (last_uid);
